-- $Id: arcanorum.lua 2955 2017-06-27 18:44:07Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

newEntity {
  base = 'ESSENCE_BASE',
  define_as = 'ALCHEMIC_ARCANORUM',
  name = 'ALCHEMIC_ARCANORUM',
  display_name = '& Alchemic Arcanorum~',
  desc = [[A shimmering glass orb, slightly larger than the Soothsayer's crystal ball.  As an Alchemist, you can use this to store and collect the powers of the Essences you find or accumulate in your travels.]],
  sval = 255,
  image = 'object/alchemic_arcanorum.png',
  level = 1,
  weight = 5,
  encumber = 5,
  cost = 5000,
  stacking = table.NIL_MERGE,
  special_gene = true,
  flags = { ATTR_MULTI=true, FULL_NAME=true, ACTIVATE=true, ACTIVATE_NO_WIELD=true, EASY_USE=true },
  ignore_dam = { [DamageType.COLD]=true, [DamageType.ELEC]=true, [DamageType.FIRE]=true, [DamageType.ACID]=true },
  pval = 200,
  desc_post = function(self, t)
    t = t or {}
    if t.chardump or not self:isIDd() then return nil end
    if not game.zone then return end
    local lines = { 'Contains essences:' }
    local kk = table.keys(self.essences)
    if #kk <= 0 then
      return '\n#LIGHT_BLUE#Contains no essences#LAST#'
    end
    table.sort(kk)
    for _, k in ipairs(kk) do
      local n = self.essences[k]
      local o = game.zone.object_list[k]
      if o and n > 0 then
	lines[#lines+1] = ('%3d %s'):format(n, o.display_name)
      end
    end
    lines = table.concat(lines, '\n· ')
    return '\n#LIGHT_BLUE#'..lines..'#LAST#'
  end,
  essences = {},
  use_simple = {
    use = function(self, who, inven, item)
      local list = {
	{ name='Absorb essences', action='absorb' },
	{ name='Extract essences', action='extract' },
      }
      local function cb(item)
	who:talentDialogReturn(item and item.action)
      end
      local d = ui_util.listPopup('Alchemic Arcanorum', 'Select function:', list, 500, cb)
      local act = who:talentDialog(d)
      local used = true
      if act == 'extract' then
	local ExtractEssences = require 'mod.dialogs.ExtractEssencesDialog'
	used = who:talentDialog(ExtractEssences.new(self, who))
      elseif act == 'absorb' then
	local inven = who:getInven(who.INVEN_INVEN)
	-- Walk the inventory in reverse, since we're going to be removing
	-- things from it.
	for i = #inven, 1, -1 do
	  local o = inven[i]
	  if o.type == 'misc' and o.subtype == 'alchemic' and o.define_as ~= 'ALCHEMIC_ARCANORUM' and not o.quest_obj then
	    local oo = who:removeObject(who.INVEN_INVEN, i, true)
	    game.logPlayer(who, 'Your %s absorbs %s.', self:getName{no_count=true}, oo:getName())
	    self.essences[oo.define_as] = (self.essences[oo.define_as] or 0) + oo:getNumber()
	    if game:hasEntity(oo) then game:removeEntity(oo) end
	  end
	end
      else
	used = false
      end
      return { used=used }
    end,
    name = [[absorb essences in inventory or extract stored essences]],
  },
  on_drop = function(self, who)
    if who == game.player then
      game.logPlayer(who, [[You really shouldn't drop your %s.]], self:getName{no_count=true})
      return true
    end
  end,
}
