-- $Id: ExtractEssencesDialog.lua 2862 2017-05-08 00:45:49Z dsb $
-- ToME - Tales of Maj'Eyal
-- Copyright © 2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham ("Zizzo")
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local Button = require 'engine.ui.Button'
local ListColumns = require 'engine.ui.ListColumns'
local Textzone = require 'engine.ui.Textzone'

module(..., package.seeall, class.inherit(Dialog))

local instructions = [[Keyboard: #00FF00#up/down arrow#LAST# to select an essence; #00FF00#left/right arrow#LAST# to decrease/increase number of that essence to extract.
Mouse: #00FF00#left/right click#LAST# on an essence to increase/decrease number of that essence to extract.
#00FF00#RETURN#LAST# to accept, #00FF00#ESC#LAST# to cancel.]]

function _M:init(arc, who)
  self.arcanorum = arc
  self.actor = who
  Dialog.init(self, 'Alchemic Arcanorum', 500, math.floor(game.h*0.8))

  self.c_tut = Textzone.new {
    width = self.iw - 10,
    auto_height = true,
    no_color_bleed = true,
    text = instructions,
  }

  self.c_ok = Button.new {
    text = 'OK',
    fct = function() self:applyDialog(true) end,
  }
  self.c_cancel = Button.new {
    text = 'Cancel',
    fct = function() self:applyDialog(false) end,
  }

  local list = table.keys(self.arcanorum.essences)
  table.sort(list)
  for i, v in ipairs(list) do
    list[i] = {
      tag = v,
      name = game.zone.object_list[v].display_name,
      num = 0,
      max = self.arcanorum.essences[v],
    }
  end
  self.list = list

  self.c_list= ListColumns.new {
    width = self.iw - 10,
    height = self.ih - self.c_tut.h - self.c_ok.h - 10,
    columns = {
      { name='Essence', width=70, display_prop='name' },
      { name='#', width=15, display_prop='num' },
      { name='Max', width=15, display_prop='max' },
    },
    sortable = true,
    scrollbar = true,
    list = self.list,
    all_clicks = true,
    fct = function(row, sel, btn, evt)
      if evt == 'button' then
	self:adjSelected(btn == 'right' and -1 or 1)
      elseif evt == 'key' then
	-- Pass through to the dialog's key handler.
	self.key:triggerVirtual('ACCEPT')
      end
    end,
  }

  self:loadUI {
    { left=0, top=0, ui=self.c_tut },
    { left=0, top=self.c_tut, ui=self.c_list },
    { right=0, bottom=0, ui=self.c_cancel },
    { right=self.c_cancel.w, bottom=0, ui=self.c_ok },
  }
  self:setFocus(self.c_list)
  self:setupUI()

  self.key:addBinds {
    ACCEPT = function() self:applyDialog(true) end,
    EXIT = function() self:applyDialog(false) end,
  }
  self.c_list.key:addBinds {
    MOVE_LEFT = function() self:adjSelected(-1) end,
    MOVE_RIGHT = function() self:adjSelected(1) end,
  }
end

function _M:adjSelected(delta)
  local item = self.c_list.sel and self.list[self.c_list.sel]
  if item then
    item.num = util.bound(item.num + delta, 0, item.max)
    self.c_list:generateRow(self.list[self.c_list.sel], true)
  end
end

function _M:applyDialog(extract)
  local any = false
  for _, row in ipairs(self.list) do any = any or row.num > 0 end
  if extract and any then
    local ess = self.arcanorum.essences
    for _, row in ipairs(self.list) do
      if row.num > 0 then
	local o = game.zone:makeEntityByName(game.level, 'object', row.tag)
	if o then
	  o:setNumber(row.num)
	  ess[row.tag] = ess[row.tag] - row.num
	  if ess[row.tag] <= 0 then ess[row.tag] = nil end
	  game.logPlayer(self.actor, 'You extract from your %s:  %s', self.arcanorum:getName{no_count=true}, o:getName())
	  self.actor:addOrDropObject('INVEN', o, nil, true)
	end
      end
    end
    self.actor:sortInven()
  end
  self.actor:talentDialogReturn(extract and any)
  game:unregisterDialog(self)
end
